@@ -5,7 +5,7 @@
 
     The Sphinx documentation toolchain.
 
-    :copyright: Copyright 2007-2011 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2013 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
@@ -15,8 +15,12 @@
 import sys
 from os import path
 
-__version__  = '1.1.1'
-__released__ = '1.1.1'  # used when Sphinx builds its own docs
+__version__  = '1.2'
+__released__ = '1.2'  # used when Sphinx builds its own docs
+# version info for better programmatic use
+# possible values for 3rd element: 'alpha', 'beta', 'rc', 'final'
+# 'final' has 0 as the last element
+version_info = (1, 2, 0, 'final', 0)
 
 package_dir = path.abspath(path.dirname(__file__))
 
@@ -38,11 +42,9 @@ if '+' in __version__ or 'pre' in __version__:
 
 def main(argv=sys.argv):
     """Sphinx build "main" command-line entry."""
-    if sys.version_info[:3] < (2, 4, 0):
-        sys.stderr.write('Error: Sphinx requires at least '
-                         'Python 2.4 to run.\n')
+    if sys.version_info[:3] < (2, 5, 0):
+        sys.stderr.write('Error: Sphinx requires at least Python 2.5 to run.\n')
         return 1
-
     try:
         from sphinx import cmdline
     except ImportError:
@@ -69,6 +71,12 @@ def main(argv=sys.argv):
                 sys.stderr.write(hint)
             return 1
         raise
+    if sys.version_info[:3] >= (3, 3, 0):
+        from sphinx.util.compat import docutils_version
+        if docutils_version < (0, 10):
+            sys.stderr.write('Error: Sphinx requires at least '
+                             'Docutils 0.10 for Python 3.3 and above.\n')
+            return 1
     return cmdline.main(argv)
 
 
