@@ -3,29 +3,35 @@
     sphinx.directives.other
     ~~~~~~~~~~~~~~~~~~~~~~~
 
-    :copyright: Copyright 2007-2011 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2013 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
-import os
-
 from docutils import nodes
 from docutils.parsers.rst import Directive, directives
+from docutils.parsers.rst.directives.admonitions import BaseAdmonition
+from docutils.parsers.rst.directives.misc import Class
+from docutils.parsers.rst.directives.misc import Include as BaseInclude
 
 from sphinx import addnodes
-from sphinx.locale import pairindextypes, _
+from sphinx.locale import versionlabels, _
 from sphinx.util import url_re, docname_join
-from sphinx.util.nodes import explicit_title_re
-from sphinx.util.compat import make_admonition
+from sphinx.util.nodes import explicit_title_re, set_source_info, \
+    process_index_entry
 from sphinx.util.matching import patfilter
 
 
+def int_or_nothing(argument):
+    if not argument:
+        return 999
+    return int(argument)
+
+
 class TocTree(Directive):
     """
     Directive to notify Sphinx about the hierarchical structure of the docs,
     and to include a table-of-contents like tree in the current document.
     """
-
     has_content = True
     required_arguments = 0
     optional_arguments = 0
@@ -34,7 +40,8 @@ class TocTree(Directive):
         'maxdepth': int,
         'glob': directives.flag,
         'hidden': directives.flag,
-        'numbered': directives.flag,
+        'includehidden': directives.flag,
+        'numbered': int_or_nothing,
         'titlesonly': directives.flag,
     }
 
@@ -73,8 +80,9 @@ class TocTree(Directive):
                     entries.append((title, ref))
                 elif docname not in env.found_docs:
                     ret.append(self.state.document.reporter.warning(
-                        'toctree references unknown document %r' % docname,
-                        line=self.lineno))
+                        'toctree contains reference to nonexisting '
+                        'document %r' % docname, line=self.lineno))
+                    env.note_reread()
                 else:
                     entries.append((title, docname))
                     includefiles.append(docname)
@@ -98,8 +106,10 @@ class TocTree(Directive):
         subnode['maxdepth'] = self.options.get('maxdepth', -1)
         subnode['glob'] = glob
         subnode['hidden'] = 'hidden' in self.options
-        subnode['numbered'] = 'numbered' in self.options
+        subnode['includehidden'] = 'includehidden' in self.options
+        subnode['numbered'] = self.options.get('numbered', 0)
         subnode['titlesonly'] = 'titlesonly' in self.options
+        set_source_info(self, subnode)
         wrappernode = nodes.compound(classes=['toctree-wrapper'])
         wrappernode.append(subnode)
         ret.append(wrappernode)
@@ -111,7 +121,6 @@ class Author(Directive):
     Directive to give the name of the author of the current document
     or section. Shown in the output only if the show_authors option is on.
     """
-
     has_content = False
     required_arguments = 1
     optional_arguments = 0
@@ -144,17 +153,12 @@ class Index(Directive):
     """
     Directive to add entries to the index.
     """
-
     has_content = False
     required_arguments = 1
     optional_arguments = 0
     final_argument_whitespace = True
     option_spec = {}
 
-    indextypes = [
-        'single', 'pair', 'double', 'triple',
-    ]
-
     def run(self):
         arguments = self.arguments[0].split('\n')
         env = self.state.document.settings.env
@@ -163,29 +167,10 @@ class Index(Directive):
         self.state.document.note_explicit_target(targetnode)
         indexnode = addnodes.index()
         indexnode['entries'] = ne = []
+        indexnode['inline'] = False
+        set_source_info(self, indexnode)
         for entry in arguments:
-            entry = entry.strip()
-            for type in pairindextypes:
-                if entry.startswith(type+':'):
-                    value = entry[len(type)+1:].strip()
-                    value = pairindextypes[type] + '; ' + value
-                    ne.append(('pair', value, targetid, value))
-                    break
-            else:
-                for type in self.indextypes:
-                    if entry.startswith(type+':'):
-                        value = entry[len(type)+1:].strip()
-                        if type == 'double':
-                            type = 'pair'
-                        ne.append((type, value, targetid, value))
-                        break
-                # shorthand notation for single entries
-                else:
-                    for value in entry.split(','):
-                        value = value.strip()
-                        if not value:
-                            continue
-                        ne.append(('single', value, targetid, value))
+            ne.extend(process_index_entry(entry, targetid))
         return [indexnode, targetnode]
 
 
@@ -193,7 +178,6 @@ class VersionChange(Directive):
     """
     Directive to describe a change/addition/deprecation in a specific version.
     """
-
     has_content = True
     required_arguments = 1
     optional_arguments = 1
@@ -203,53 +187,50 @@ class VersionChange(Directive):
     def run(self):
         node = addnodes.versionmodified()
         node.document = self.state.document
+        set_source_info(self, node)
         node['type'] = self.name
         node['version'] = self.arguments[0]
+        text = versionlabels[self.name] % self.arguments[0]
         if len(self.arguments) == 2:
             inodes, messages = self.state.inline_text(self.arguments[1],
                                                       self.lineno+1)
-            node.extend(inodes)
-            if self.content:
-                self.state.nested_parse(self.content, self.content_offset, node)
-            ret = [node] + messages
+            para = nodes.paragraph(self.arguments[1], '', *inodes)
+            set_source_info(self, para)
+            node.append(para)
         else:
-            ret = [node]
+            messages = []
+        if self.content:
+            self.state.nested_parse(self.content, self.content_offset, node)
+        if len(node):
+            if isinstance(node[0], nodes.paragraph) and node[0].rawsource:
+                content = nodes.inline(node[0].rawsource, translatable=True)
+                content.source = node[0].source
+                content.line = node[0].line
+                content += node[0].children
+                node[0].replace_self(nodes.paragraph('', '', content))
+            node[0].insert(0, nodes.inline('', '%s: ' % text,
+                                           classes=['versionmodified']))
+        else:
+            para = nodes.paragraph('', '',
+                nodes.inline('', '%s.' % text, classes=['versionmodified']))
+            node.append(para)
         env = self.state.document.settings.env
-        env.note_versionchange(node['type'], node['version'], node, self.lineno)
-        return ret
+        # XXX should record node.source as well
+        env.note_versionchange(node['type'], node['version'], node, node.line)
+        return [node] + messages
 
 
-class SeeAlso(Directive):
+class SeeAlso(BaseAdmonition):
     """
     An admonition mentioning things to look at as reference.
     """
-
-    has_content = True
-    required_arguments = 0
-    optional_arguments = 1
-    final_argument_whitespace = True
-    option_spec = {}
-
-    def run(self):
-        ret = make_admonition(
-            addnodes.seealso, self.name, [_('See also')], self.options,
-            self.content, self.lineno, self.content_offset, self.block_text,
-            self.state, self.state_machine)
-        if self.arguments:
-            argnodes, msgs = self.state.inline_text(self.arguments[0],
-                                                    self.lineno)
-            para = nodes.paragraph()
-            para += argnodes
-            para += msgs
-            ret[0].insert(1, para)
-        return ret
+    node_class = addnodes.seealso
 
 
 class TabularColumns(Directive):
     """
     Directive to give an explicit tabulary column definition to LaTeX.
     """
-
     has_content = False
     required_arguments = 1
     optional_arguments = 0
@@ -259,6 +240,7 @@ class TabularColumns(Directive):
     def run(self):
         node = addnodes.tabular_col_spec()
         node['spec'] = self.arguments[0]
+        set_source_info(self, node)
         return [node]
 
 
@@ -266,7 +248,6 @@ class Centered(Directive):
     """
     Directive to create a centered line of bold text.
     """
-
     has_content = False
     required_arguments = 1
     optional_arguments = 0
@@ -283,12 +264,10 @@ class Centered(Directive):
         return [subnode] + messages
 
 
-
 class Acks(Directive):
     """
     Directive for a list of names.
     """
-
     has_content = True
     required_arguments = 0
     optional_arguments = 0
@@ -310,7 +289,6 @@ class HList(Directive):
     """
     Directive for a list that gets compacted horizontally.
     """
-
     has_content = True
     required_arguments = 0
     optional_arguments = 0
@@ -347,7 +325,6 @@ class Only(Directive):
     """
     Directive to only include text if the given tag(s) are enabled.
     """
-
     has_content = True
     required_arguments = 1
     optional_arguments = 0
@@ -357,26 +334,64 @@ class Only(Directive):
     def run(self):
         node = addnodes.only()
         node.document = self.state.document
-        node.line = self.lineno
+        set_source_info(self, node)
         node['expr'] = self.arguments[0]
-        self.state.nested_parse(self.content, self.content_offset, node,
-                                match_titles=1)
-        return [node]
 
+        # Same as util.nested_parse_with_titles but try to handle nested
+        # sections which should be raised higher up the doctree.
+        surrounding_title_styles = self.state.memo.title_styles
+        surrounding_section_level = self.state.memo.section_level
+        self.state.memo.title_styles = []
+        self.state.memo.section_level = 0
+        try:
+            self.state.nested_parse(self.content, self.content_offset,
+                                    node, match_titles=1)
+            title_styles = self.state.memo.title_styles
+            if (not surrounding_title_styles
+                or not title_styles
+                or title_styles[0] not in surrounding_title_styles
+                or not self.state.parent):
+                # No nested sections so no special handling needed.
+                return [node]
+            # Calculate the depths of the current and nested sections.
+            current_depth = 0
+            parent = self.state.parent
+            while parent:
+                current_depth += 1
+                parent = parent.parent
+            current_depth -= 2
+            title_style = title_styles[0]
+            nested_depth = len(surrounding_title_styles)
+            if title_style in surrounding_title_styles:
+                nested_depth = surrounding_title_styles.index(title_style)
+            # Use these depths to determine where the nested sections should
+            # be placed in the doctree.
+            n_sects_to_raise = current_depth - nested_depth + 1
+            parent = self.state.parent
+            for i in xrange(n_sects_to_raise):
+                if parent.parent:
+                    parent = parent.parent
+            parent.append(node)
+            return []
+        finally:
+            self.state.memo.title_styles = surrounding_title_styles
+            self.state.memo.section_level = surrounding_section_level
 
-from docutils.parsers.rst.directives.misc import Include as BaseInclude
 
 class Include(BaseInclude):
     """
     Like the standard "Include" directive, but interprets absolute paths
-    correctly.
+    "correctly", i.e. relative to source directory.
     """
 
     def run(self):
-        if self.arguments[0].startswith('/') or \
-               self.arguments[0].startswith(os.sep):
-            env = self.state.document.settings.env
-            self.arguments[0] = os.path.join(env.srcdir, self.arguments[0][1:])
+        env = self.state.document.settings.env
+        if self.arguments[0].startswith('<') and \
+           self.arguments[0].endswith('>'):
+            # docutils "standard" includes, do not do path processing
+            return BaseInclude.run(self)
+        rel_filename, filename = env.relfn2path(self.arguments[0])
+        self.arguments[0] = filename
         return BaseInclude.run(self)
 
 
@@ -397,7 +412,6 @@ directives.register_directive('only', Only)
 directives.register_directive('include', Include)
 
 # register the standard rst class directive under a different name
-from docutils.parsers.rst.directives.misc import Class
 # only for backwards compatibility now
 directives.register_directive('cssclass', Class)
 # new standard name when default-domain with "class" is in effect
