@@ -5,28 +5,41 @@
 
     docutils writers handling Sphinx' custom nodes.
 
-    :copyright: Copyright 2007-2011 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2013 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
 import sys
 import posixpath
 import os
+import copy
 
 from docutils import nodes
 from docutils.writers.html4css1 import Writer, HTMLTranslator as BaseTranslator
 
 from sphinx import addnodes
-from sphinx.locale import admonitionlabels, versionlabels, _
+from sphinx.locale import admonitionlabels, _
 from sphinx.util.smartypants import sphinx_smarty_pants
 
 try:
-    import Image                        # check for the Python Imaging Library
+    from PIL import Image        # check for the Python Imaging Library
 except ImportError:
-    Image = None
+    try:
+        import Image
+    except ImportError:
+        Image = None
 
+# A good overview of the purpose behind these classes can be found here:
+# http://www.arnebrodowski.de/blog/write-your-own-restructuredtext-writer.html
 
 class HTMLWriter(Writer):
+
+    # override embed-stylesheet default value to 0.
+    settings_spec = copy.deepcopy(Writer.settings_spec)
+    for _setting in settings_spec[2]:
+        if '--embed-stylesheet' in _setting[1]:
+            _setting[2]['default'] = 0
+
     def __init__(self, builder):
         Writer.__init__(self)
         self.builder = builder
@@ -59,8 +72,14 @@ class HTMLTranslator(BaseTranslator):
         self.highlightlang = builder.config.highlight_language
         self.highlightlinenothreshold = sys.maxint
         self.protect_literal_text = 0
-        self.add_permalinks = builder.config.html_add_permalinks
+        self.permalink_text = builder.config.html_add_permalinks
+        # support backwards-compatible setting to a bool
+        if not isinstance(self.permalink_text, basestring):
+            self.permalink_text = self.permalink_text and u'\u00B6' or ''
+        self.permalink_text = self.encode(self.permalink_text)
         self.secnumber_suffix = builder.config.html_secnumber_suffix
+        self.param_separator = ''
+        self._table_row_index = 0
 
     def visit_start_of_file(self, node):
         # only occurs in the single-file builder
@@ -81,11 +100,12 @@ class HTMLTranslator(BaseTranslator):
                and node['ids'] and node['first']:
             self.body.append('<!--[%s]-->' % node['ids'][0])
     def depart_desc_signature(self, node):
-        if node['ids'] and self.add_permalinks and self.builder.add_permalinks:
+        if node['ids'] and self.permalink_text and self.builder.add_permalinks:
             self.body.append(u'<a class="headerlink" href="#%s" '
                              % node['ids'][0] +
-                             u'title="%s">\u00B6</a>' %
-                             _('Permalink to this definition'))
+                             u'title="%s">%s</a>' % (
+                             _('Permalink to this definition'),
+                             self.permalink_text))
         self.body.append('</dt>\n')
 
     def visit_desc_addname(self, node):
@@ -111,12 +131,13 @@ class HTMLTranslator(BaseTranslator):
     def visit_desc_parameterlist(self, node):
         self.body.append('<big>(</big>')
         self.first_param = 1
+        self.param_separator = node.child_text_separator
     def depart_desc_parameterlist(self, node):
         self.body.append('<big>)</big>')
 
     def visit_desc_parameter(self, node):
         if not self.first_param:
-            self.body.append(', ')
+            self.body.append(self.param_separator)
         else:
             self.first_param = 0
         if not node.hasattr('noemph'):
@@ -140,21 +161,10 @@ class HTMLTranslator(BaseTranslator):
     def depart_desc_content(self, node):
         self.body.append('</dd>')
 
-    def visit_refcount(self, node):
-        self.body.append(self.starttag(node, 'em', '', CLASS='refcount'))
-    def depart_refcount(self, node):
-        self.body.append('</em>')
-
     def visit_versionmodified(self, node):
-        self.body.append(self.starttag(node, 'p', CLASS=node['type']))
-        text = versionlabels[node['type']] % node['version']
-        if len(node):
-            text += ': '
-        else:
-            text += '.'
-        self.body.append('<span class="versionmodified">%s</span>' % text)
+        self.body.append(self.starttag(node, 'div', CLASS=node['type']))
     def depart_versionmodified(self, node):
-        self.body.append('</p>\n')
+        self.body.append('</div>\n')
 
     # overwritten
     def visit_reference(self, node):
@@ -180,7 +190,7 @@ class HTMLTranslator(BaseTranslator):
             atts['title'] = node['reftitle']
         self.body.append(self.starttag(node, 'a', '', **atts))
 
-        if node.hasattr('secnumber'):
+        if node.get('secnumber'):
             self.body.append(('%s' + self.secnumber_suffix) %
                              '.'.join(map(str, node['secnumber'])))
 
@@ -192,7 +202,7 @@ class HTMLTranslator(BaseTranslator):
     def visit_admonition(self, node, name=''):
         self.body.append(self.starttag(
             node, 'div', CLASS=('admonition ' + name)))
-        if name and name != 'seealso':
+        if name:
             node.insert(0, nodes.title(name, admonitionlabels[name]))
         self.set_first_last(node)
 
@@ -202,14 +212,14 @@ class HTMLTranslator(BaseTranslator):
         self.depart_admonition(node)
 
     def add_secnumber(self, node):
-        if node.hasattr('secnumber'):
+        if node.get('secnumber'):
             self.body.append('.'.join(map(str, node['secnumber'])) +
                              self.secnumber_suffix)
         elif isinstance(node.parent, nodes.section):
             anchorname = '#' + node.parent['ids'][0]
             if anchorname not in self.builder.secnumbers:
                 anchorname = ''  # try first heading which has no anchor
-            if anchorname in self.builder.secnumbers:
+            if self.builder.secnumbers.get(anchorname):
                 numbers = self.builder.secnumbers[anchorname]
                 self.body.append('.'.join(map(str, numbers)) +
                                  self.secnumber_suffix)
@@ -227,15 +237,18 @@ class HTMLTranslator(BaseTranslator):
         lang = self.highlightlang
         linenos = node.rawsource.count('\n') >= \
                   self.highlightlinenothreshold - 1
+        highlight_args = node.get('highlight_args', {})
         if node.has_key('language'):
             # code-block directives
             lang = node['language']
+            highlight_args['force'] = True
         if node.has_key('linenos'):
             linenos = node['linenos']
         def warner(msg):
             self.builder.warn(msg, (self.builder.current_docname, node.line))
         highlighted = self.highlighter.highlight_block(
-            node.rawsource, lang, linenos, warn=warner)
+            node.rawsource, lang, warn=warner, linenos=linenos,
+            **highlight_args)
         starttag = self.starttag(node, 'div', suffix='',
                                  CLASS='highlight-%s' % lang)
         self.body.append(starttag + highlighted + '</div>\n')
@@ -244,11 +257,14 @@ class HTMLTranslator(BaseTranslator):
     def visit_doctest_block(self, node):
         self.visit_literal_block(node)
 
+    # overwritten to add the <div> (for XHTML compliance)
+    def visit_block_quote(self, node):
+        self.body.append(self.starttag(node, 'blockquote') + '<div>')
+    def depart_block_quote(self, node):
+        self.body.append('</div></blockquote>\n')
+
     # overwritten
     def visit_literal(self, node):
-        if len(node.children) == 1 and \
-               node.children[0] in ('None', 'True', 'False'):
-            node['classes'].append('xref')
         self.body.append(self.starttag(node, 'tt', '',
                                        CLASS='docutils literal'))
         self.protect_literal_text += 1
@@ -262,12 +278,13 @@ class HTMLTranslator(BaseTranslator):
         for production in node:
             names.append(production['tokenname'])
         maxlen = max(len(name) for name in names)
+        lastname = None
         for production in node:
             if production['tokenname']:
                 lastname = production['tokenname'].ljust(maxlen)
                 self.body.append(self.starttag(production, 'strong', ''))
                 self.body.append(lastname + '</strong> ::= ')
-            else:
+            elif lastname is not None:
                 self.body.append('%s     ' % (' '*len(lastname)))
             production.walkabout(self)
             self.body.append('\n')
@@ -293,6 +310,9 @@ class HTMLTranslator(BaseTranslator):
         if isinstance(node.parent, addnodes.desc_content):
             # Never compact desc_content items.
             return False
+        if isinstance(node.parent, addnodes.versionmodified):
+            # Never compact versionmodified nodes.
+            return False
         return BaseTranslator.should_be_compact_paragraph(self, node)
 
     def visit_compact_paragraph(self, node):
@@ -327,25 +347,26 @@ class HTMLTranslator(BaseTranslator):
 
         if node['uri'].lower().endswith('svg') or \
            node['uri'].lower().endswith('svgz'):
-            atts = {'data': node['uri'], 'type': 'image/svg+xml'}
+            atts = {'src': node['uri']}
             if node.has_key('width'):
                 atts['width'] = node['width']
             if node.has_key('height'):
                 atts['height'] = node['height']
+            if node.has_key('alt'):
+                atts['alt'] = node['alt']
             if node.has_key('align'):
                 self.body.append('<div align="%s" class="align-%s">' %
                                  (node['align'], node['align']))
                 self.context.append('</div>\n')
             else:
                 self.context.append('')
-            embatts = atts.copy()
-            embatts['src'] = embatts.pop('data')
-            self.body.append(self.starttag(node, 'object', '', **atts))
-            self.body.append(self.emptytag(node, 'embed', '', **embatts))
-            self.body.append('</object>\n')
+            self.body.append(self.emptytag(node, 'img', '', **atts))
             return
 
         if node.has_key('scale'):
+            # Try to figure out image height and width.  Docutils does that too,
+            # but it tries the final file name, which does not necessarily exist
+            # yet at the time the HTML file is written.
             if Image and not (node.has_key('width')
                               and node.has_key('height')):
                 try:
@@ -419,8 +440,6 @@ class HTMLTranslator(BaseTranslator):
                 encoded = self.bulk_text_processor(encoded)
             self.body.append(encoded)
 
-    # these are all for docutils 0.5 compatibility
-
     def visit_note(self, node):
         self.visit_admonition(node, 'note')
     def depart_note(self, node):
@@ -481,24 +500,57 @@ class HTMLTranslator(BaseTranslator):
     def depart_abbreviation(self, node):
         self.body.append('</abbr>')
 
+    def visit_termsep(self, node):
+        self.body.append('<br />')
+        raise nodes.SkipNode
+
     def depart_title(self, node):
         close_tag = self.context[-1]
-        if (self.add_permalinks and self.builder.add_permalinks and
+        if (self.permalink_text and self.builder.add_permalinks and
             node.parent.hasattr('ids') and node.parent['ids']):
             aname = node.parent['ids'][0]
             # add permalink anchor
             if close_tag.startswith('</h'):
                 self.body.append(u'<a class="headerlink" href="#%s" ' % aname +
-                                 u'title="%s">\u00B6</a>' %
-                                 _('Permalink to this headline'))
+                                 u'title="%s">%s</a>' % (
+                                 _('Permalink to this headline'),
+                                 self.permalink_text))
             elif close_tag.startswith('</a></h'):
                 self.body.append(u'</a><a class="headerlink" href="#%s" ' %
                                  aname +
-                                 u'title="%s">\u00B6' %
-                                 _('Permalink to this headline'))
+                                 u'title="%s">%s' % (
+                                 _('Permalink to this headline'),
+                                 self.permalink_text))
 
         BaseTranslator.depart_title(self, node)
 
+    # overwritten to add even/odd classes
+
+    def visit_table(self, node):
+        self._table_row_index = 0
+        return BaseTranslator.visit_table(self, node)
+
+    def visit_row(self, node):
+        self._table_row_index += 1
+        if self._table_row_index % 2 == 0:
+            node['classes'].append('row-even')
+        else:
+            node['classes'].append('row-odd')
+        self.body.append(self.starttag(node, 'tr', ''))
+        node.column = 0
+
+    def visit_field_list(self, node):
+        self._fieldlist_row_index = 0
+        return BaseTranslator.visit_field_list(self, node)
+
+    def visit_field(self, node):
+        self._fieldlist_row_index += 1
+        if self._fieldlist_row_index % 2 == 0:
+            node['classes'].append('field-even')
+        else:
+            node['classes'].append('field-odd')
+        self.body.append(self.starttag(node, 'tr', '', CLASS='field'))
+
     def unknown_visit(self, node):
         raise NotImplementedError('Unknown node: ' + node.__class__.__name__)
 
@@ -525,8 +577,15 @@ class SmartyPantsHTMLTranslator(HTMLTranslator):
         self.no_smarty += 1
         try:
             HTMLTranslator.visit_literal_block(self, node)
-        finally:
+        except nodes.SkipNode:
+            # HTMLTranslator raises SkipNode for simple literal blocks,
+            # but not for parsed literal blocks
             self.no_smarty -= 1
+            raise
+
+    def depart_literal_block(self, node):
+        HTMLTranslator.depart_literal_block(self, node)
+        self.no_smarty -= 1
 
     def visit_literal_emphasis(self, node):
         self.no_smarty += 1
