@@ -6,32 +6,53 @@
     Test the autodoc extension.  This tests mainly the Documenters; the auto
     directives are tested in a test source file translated by test_build.
 
-    :copyright: Copyright 2007-2011 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2013 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
-from util import *
+import sys
+from StringIO import StringIO
+
+# "raises" imported for usage by autodoc
+from util import TestApp, Struct, raises
+from nose.tools import with_setup
 
 from docutils.statemachine import ViewList
 
 from sphinx.ext.autodoc import AutoDirective, add_documenter, \
      ModuleLevelDocumenter, FunctionDocumenter, cut_lines, between, ALL
 
+app = None
 
 def setup_module():
-    global app, lid, options, directive
-
+    global app
     app = TestApp()
     app.builder.env.app = app
+    app.builder.env.temp_data['docname'] = 'dummy'
     app.connect('autodoc-process-docstring', process_docstring)
     app.connect('autodoc-process-signature', process_signature)
     app.connect('autodoc-skip-member', skip_member)
 
+
+def teardown_module():
+    app.cleanup()
+
+
+directive = options = None
+
+def setup_test():
+    global options, directive
+    global processed_docstrings, processed_signatures, _warnings
+
     options = Struct(
         inherited_members = False,
         undoc_members = False,
+        private_members = False,
+        special_members = False,
+        imported_members = False,
         show_inheritance = False,
         noindex = False,
+        annotation = None,
         synopsis = '',
         platform = '',
         deprecated = False,
@@ -48,8 +69,9 @@ def setup_module():
         filename_set = set(),
     )
 
-def teardown_module():
-    app.cleanup()
+    processed_docstrings = []
+    processed_signatures = []
+    _warnings = []
 
 
 _warnings = []
@@ -74,12 +96,15 @@ def process_signature(app, what, name, obj, options, args, retann):
 
 
 def skip_member(app, what, name, obj, skip, options):
+    if name in ('__special1__', '__special2__'):
+        return skip
     if name.startswith('_'):
         return True
     if name == 'skipmeth':
         return True
 
 
+@with_setup(setup_test)
 def test_parse_name():
     def verify(objtype, name, result):
         inst = AutoDirective._registry[objtype](directive, name)
@@ -121,6 +146,7 @@ def test_parse_name():
     del directive.env.temp_data['autodoc:class']
 
 
+@with_setup(setup_test)
 def test_format_signature():
     def formatsig(objtype, name, obj, args, retann):
         inst = AutoDirective._registry[objtype](directive, name)
@@ -159,6 +185,21 @@ def test_format_signature():
         assert formatsig('class', 'C', C, None, None) == '(a, b=None)'
     assert formatsig('class', 'C', D, 'a, b', 'X') == '(a, b) -> X'
 
+    #__init__ have signature at first line of docstring
+    class F2:
+        '''some docstring for F2.'''
+        def __init__(self, *args, **kw):
+            '''
+            __init__(a1, a2, kw1=True, kw2=False)
+
+            some docstring for __init__.
+            '''
+    class G2(F2, object):
+        pass
+    for C in (F2, G2):
+        assert formatsig('class', 'C', C, None, None) == \
+            '(a1, a2, kw1=True, kw2=False)'
+
     # test for methods
     class H:
         def foo1(self, b, *c):
@@ -176,7 +217,23 @@ def test_format_signature():
     # test processing by event handler
     assert formatsig('method', 'bar', H.foo1, None, None) == '42'
 
+    # test functions created via functools.partial
+    from functools import partial
+    curried1 = partial(lambda a, b, c: None, 'A')
+    assert formatsig('function', 'curried1', curried1, None, None) == \
+        '(b, c)'
+    curried2 = partial(lambda a, b, c=42: None, 'A')
+    assert formatsig('function', 'curried2', curried2, None, None) == \
+        '(b, c=42)'
+    curried3 = partial(lambda a, b, *c: None, 'A')
+    assert formatsig('function', 'curried3', curried3, None, None) == \
+        '(b, *c)'
+    curried4 = partial(lambda a, b, c=42, *d, **e: None, 'A')
+    assert formatsig('function', 'curried4', curried4, None, None) == \
+        '(b, c=42, *d, **e)'
 
+
+@with_setup(setup_test)
 def test_get_doc():
     def getdocl(objtype, obj, encoding=None):
         inst = AutoDirective._registry[objtype](directive, 'tmp')
@@ -244,7 +301,41 @@ def test_get_doc():
     assert getdocl('class', D) == ['Class docstring', '', 'Init docstring',
                                    '', 'Other', ' lines']
 
+    #__init__ have signature at first line of docstring
+    class E:
+        """Class docstring"""
+        def __init__(self, *args, **kw):
+            """
+            __init__(a1, a2, kw1=True, kw2=False)
 
+            Init docstring
+            """
+
+    # signature line in the docstring will be kept when
+    # autodoc_docstring_signature == False
+    directive.env.config.autodoc_docstring_signature = False
+    directive.env.config.autoclass_content = 'class'
+    assert getdocl('class', E) == ['Class docstring']
+    directive.env.config.autoclass_content = 'init'
+    assert getdocl('class', E) == ['__init__(a1, a2, kw1=True, kw2=False)',
+                                   '', 'Init docstring']
+    directive.env.config.autoclass_content = 'both'
+    assert getdocl('class', E) == ['Class docstring', '',
+                                   '__init__(a1, a2, kw1=True, kw2=False)',
+                                   '', 'Init docstring']
+
+    # signature line in the docstring will be removed when
+    # autodoc_docstring_signature == True
+    directive.env.config.autodoc_docstring_signature = True  #default
+    directive.env.config.autoclass_content = 'class'
+    assert getdocl('class', E) == ['Class docstring']
+    directive.env.config.autoclass_content = 'init'
+    assert getdocl('class', E) == ['Init docstring']
+    directive.env.config.autoclass_content = 'both'
+    assert getdocl('class', E) == ['Class docstring', '', 'Init docstring']
+
+
+@with_setup(setup_test)
 def test_docstring_processing():
     def process(objtype, name, obj):
         inst = AutoDirective._registry[objtype](directive, name)
@@ -295,6 +386,8 @@ def test_docstring_processing():
     assert process('function', 'h', h) == ['first line', 'third line', '']
     app.disconnect(lid)
 
+
+@with_setup(setup_test)
 def test_new_documenter():
     class MyDocumenter(ModuleLevelDocumenter):
         objtype = 'integer'
@@ -322,6 +415,40 @@ def test_new_documenter():
     assert_result_contains('.. py:data:: integer', 'module', 'test_autodoc')
 
 
+@with_setup(setup_test, AutoDirective._special_attrgetters.clear)
+def test_attrgetter_using():
+    def assert_getter_works(objtype, name, obj, attrs=[], **kw):
+        getattr_spy = []
+        def special_getattr(obj, name, *defargs):
+            if name in attrs:
+                getattr_spy.append((obj, name))
+                return None
+            return getattr(obj, name, *defargs)
+        AutoDirective._special_attrgetters[type] = special_getattr
+
+        del getattr_spy[:]
+        inst = AutoDirective._registry[objtype](directive, name)
+        inst.generate(**kw)
+
+        hooked_members = [s[1] for s in getattr_spy]
+        documented_members = [s[1] for s in processed_signatures]
+        for attr in attrs:
+            fullname = '.'.join((name, attr))
+            assert attr in hooked_members
+            assert fullname not in documented_members, \
+                '%r was not hooked by special_attrgetter function' % fullname
+
+    options.members = ALL
+    options.inherited_members = False
+    assert_getter_works('class', 'test_autodoc.Class', Class,
+                       ['meth'])
+
+    options.inherited_members = True
+    assert_getter_works('class', 'test_autodoc.Class', Class,
+                       ['meth', 'inheritedmeth'])
+
+
+@with_setup(setup_test)
 def test_generate():
     def assert_warns(warn_str, objtype, name, **kw):
         inst = AutoDirective._registry[objtype](directive, name)
@@ -335,6 +462,7 @@ def test_generate():
         inst = AutoDirective._registry[objtype](directive, name)
         inst.generate(**kw)
         assert directive.result
+        #print '\n'.join(directive.result)
         assert len(_warnings) == 0, _warnings
         del directive.result[:]
 
@@ -379,10 +507,10 @@ def test_generate():
     assert_warns("import for autodocumenting 'foobar'",
                  'function', 'foobar', more_content=None)
     # importing
-    assert_warns("import/find module 'test_foobar'",
+    assert_warns("failed to import module 'test_foobar'",
                  'module', 'test_foobar', more_content=None)
     # attributes missing
-    assert_warns("import/find function 'util.foobar'",
+    assert_warns("failed to import function 'foobar' from module 'util'",
                  'function', 'util.foobar', more_content=None)
 
     # test auto and given content mixing
@@ -416,18 +544,32 @@ def test_generate():
                    ('attribute', 'test_autodoc.Class.attr'),
                    ('attribute', 'test_autodoc.Class.docattr'),
                    ('attribute', 'test_autodoc.Class.udocattr'),
+                   ('attribute', 'test_autodoc.Class.mdocattr'),
                    ('attribute', 'test_autodoc.Class.inst_attr_comment'),
-                   ('attribute', 'test_autodoc.Class.inst_attr_string')
+                   ('attribute', 'test_autodoc.Class.inst_attr_inline'),
+                   ('attribute', 'test_autodoc.Class.inst_attr_string'),
+                   ('method', 'test_autodoc.Class.moore'),
                    ])
     options.members = ALL
     assert_processes(should, 'class', 'Class')
     options.undoc_members = True
-    should.append(('method', 'test_autodoc.Class.undocmeth'))
+    should.extend((('attribute', 'test_autodoc.Class.skipattr'),
+                   ('method', 'test_autodoc.Class.undocmeth'),
+                   ('method', 'test_autodoc.Class.roger')))
     assert_processes(should, 'class', 'Class')
     options.inherited_members = True
     should.append(('method', 'test_autodoc.Class.inheritedmeth'))
     assert_processes(should, 'class', 'Class')
 
+    # test special members
+    options.special_members = ['__special1__']
+    should.append(('method', 'test_autodoc.Class.__special1__'))
+    assert_processes(should, 'class', 'Class')
+    options.special_members = ALL
+    should.append(('method', 'test_autodoc.Class.__special2__'))
+    assert_processes(should, 'class', 'Class')
+    options.special_members = False
+
     options.members = []
     # test module flags
     assert_result_contains('.. py:module:: test_autodoc',
@@ -484,17 +626,89 @@ def test_generate():
                   '   .. py:attribute:: Class.prop',
                   '   .. py:attribute:: Class.docattr',
                   '   .. py:attribute:: Class.udocattr',
+                  '   .. py:attribute:: Class.mdocattr',
+                  '   .. py:classmethod:: Class.roger(a, e=5, f=6)',
+                  '   .. py:classmethod:: Class.moore(a, e, f) -> happiness',
                   '   .. py:attribute:: Class.inst_attr_comment',
                   '   .. py:attribute:: Class.inst_attr_string',
                   '   .. py:method:: Class.inheritedmeth()',
                   ],
                  'class', 'Class', member_order='bysource', all_members=True)
+    del directive.env.temp_data['py:module']
 
+    # test attribute initialized to class instance from other module
+    directive.env.temp_data['autodoc:class'] = 'test_autodoc.Class'
+    assert_result_contains(u'   should be documented as well - s\xfc\xdf',
+                           'attribute', 'mdocattr')
+    del directive.env.temp_data['autodoc:class']
+
+    # test autodoc_docstring_signature
+    assert_result_contains(
+        '.. py:method:: DocstringSig.meth(FOO, BAR=1) -> BAZ', 'method',
+        'test_autodoc.DocstringSig.meth')
+    assert_result_contains(
+        '   rest of docstring', 'method', 'test_autodoc.DocstringSig.meth')
+    assert_result_contains(
+        '.. py:method:: DocstringSig.meth2()', 'method',
+        'test_autodoc.DocstringSig.meth2')
+    assert_result_contains(
+        '       indented line', 'method',
+        'test_autodoc.DocstringSig.meth2')
+    assert_result_contains(
+        '.. py:classmethod:: Class.moore(a, e, f) -> happiness', 'method',
+        'test_autodoc.Class.moore')
+
+    # test new attribute documenter behavior
+    directive.env.temp_data['py:module'] = 'test_autodoc'
+    options.undoc_members = True
+    assert_processes([('class', 'test_autodoc.AttCls'),
+                      ('attribute', 'test_autodoc.AttCls.a1'),
+                      ('attribute', 'test_autodoc.AttCls.a2'),
+                      ], 'class', 'AttCls')
+    assert_result_contains(
+        '   :annotation: = hello world', 'attribute', 'AttCls.a1')
+    assert_result_contains(
+        '   :annotation: = None', 'attribute', 'AttCls.a2')
+
+    # test explicit members with instance attributes
+    del directive.env.temp_data['autodoc:class']
+    del directive.env.temp_data['autodoc:module']
+    directive.env.temp_data['py:module'] = 'test_autodoc'
+    options.inherited_members = False
+    options.undoc_members = False
+    options.members = ALL
+    assert_processes([
+        ('class', 'test_autodoc.InstAttCls'),
+        ('attribute', 'test_autodoc.InstAttCls.ca1'),
+        ('attribute', 'test_autodoc.InstAttCls.ca2'),
+        ('attribute', 'test_autodoc.InstAttCls.ca3'),
+        ('attribute', 'test_autodoc.InstAttCls.ia1'),
+        ('attribute', 'test_autodoc.InstAttCls.ia2'),
+    ], 'class', 'InstAttCls')
+    del directive.env.temp_data['autodoc:class']
+    del directive.env.temp_data['autodoc:module']
+    options.members = ['ca1', 'ia1']
+    assert_processes([
+        ('class', 'test_autodoc.InstAttCls'),
+        ('attribute', 'test_autodoc.InstAttCls.ca1'),
+        ('attribute', 'test_autodoc.InstAttCls.ia1'),
+    ], 'class', 'InstAttCls')
+    del directive.env.temp_data['autodoc:class']
+    del directive.env.temp_data['autodoc:module']
+    del directive.env.temp_data['py:module']
+
+    # test descriptor class documentation
+    options.members = ['CustomDataDescriptor']
+    assert_result_contains('.. py:class:: CustomDataDescriptor(doc)',
+                           'module', 'test_autodoc')
+    assert_result_contains('   .. py:method:: CustomDataDescriptor.meth()',
+                           'module', 'test_autodoc')
 
 # --- generate fodder ------------
 
 __all__ = ['Class']
 
+#: documentation for the integer
 integer = 1
 
 class CustomEx(Exception):
@@ -514,6 +728,25 @@ class CustomDataDescriptor(object):
             return self
         return 42
 
+    def meth(self):
+        """Function."""
+        return "The Answer"
+
+def _funky_classmethod(name, b, c, d, docstring=None):
+    """Generates a classmethod for a class from a template by filling out
+    some arguments."""
+    def template(cls, a, b, c, d=4, e=5, f=6):
+        return a, b, c, d, e, f
+    if sys.version_info >= (2, 5):
+        from functools import partial
+        function = partial(template, b=b, c=c, d=d)
+    else:
+        def function(cls, a, e=5, f=6):
+            return template(a, b, c, d, e, f)
+    function.__name__ = name
+    function.__doc__ = docstring
+    return classmethod(function)
+
 class Base(object):
     def inheritedmeth(self):
         """Inherited function."""
@@ -552,12 +785,29 @@ class Class(Base):
     udocattr = 'quux'
     u"""should be documented as well - süß"""
 
+    # initialized to any class imported from another module
+    mdocattr = StringIO()
+    """should be documented as well - süß"""
+
+    roger = _funky_classmethod("roger", 2, 3, 4)
+
+    moore = _funky_classmethod("moore", 9, 8, 7,
+        docstring="moore(a, e, f) -> happiness")
+
     def __init__(self, arg):
+        self.inst_attr_inline = None #: an inline documented instance attr
         #: a documented instance attribute
         self.inst_attr_comment = None
         self.inst_attr_string = None
         """a documented instance attribute"""
 
+    def __special1__(self):
+        """documented special method"""
+
+    def __special2__(self):
+        # undocumented special method
+        pass
+
 
 class CustomDict(dict):
     """Docstring."""
@@ -580,3 +830,46 @@ class Outer(object):
 
     # should be documented as an alias
     factory = dict
+
+
+class DocstringSig(object):
+    def meth(self):
+        """meth(FOO, BAR=1) -> BAZ
+First line of docstring
+
+        rest of docstring
+        """
+
+    def meth2(self):
+        """First line, no signature
+        Second line followed by indentation::
+
+            indented line
+        """
+
+class StrRepr(str):
+    def __repr__(self):
+        return self
+
+class AttCls(object):
+    a1 = StrRepr('hello\nworld')
+    a2 = None
+
+class InstAttCls(object):
+    """Class with documented class and instance attributes."""
+
+    #: Doc comment for class attribute InstAttCls.ca1.
+    #: It can have multiple lines.
+    ca1 = 'a'
+
+    ca2 = 'b'    #: Doc comment for InstAttCls.ca2. One line only.
+
+    ca3 = 'c'
+    """Docstring for class attribute InstAttCls.ca3."""
+
+    def __init__(self):
+        #: Doc comment for instance attribute InstAttCls.ia1
+        self.ia1 = 'd'
+
+        self.ia2 = 'e'
+        """Docstring for instance attribute InstAttCls.ia2."""
